#ifndef PYTHONIC_INCLUDE_OPERATOR_IOR__HPP
#define PYTHONIC_INCLUDE_OPERATOR_IOR__HPP

#include "pythonic/include/operator_/ior.hpp"

PYTHONIC_NS_BEGIN

namespace operator_
{
  USING_FUNCTOR(__ior__, ior);
}
PYTHONIC_NS_END

#endif
